--
-- DE2 top-level module that includes the simple audio component
--
-- Stephen A. Edwards, Columbia University, sedwards@cs.columbia.edu
--
-- From an original by Terasic Technology, Inc.
-- (DE2_TOP.v, part of the DE2 system board CD supplied by Altera)
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity guitar_effects_top is

  port (
    -- Clocks
    
    CLOCK_27,                                      -- 27 MHz
    CLOCK_50,                                      -- 50 MHz
    EXT_CLOCK : in std_logic;                      -- External Clock

    -- Buttons and switches
    
    KEY : in std_logic_vector(3 downto 0);         -- Push buttons
    SW : in std_logic_vector(17 downto 0);         -- DPDT switches

    -- LED displays

    HEX0, HEX1, HEX2, HEX3, HEX4, HEX5, HEX6, HEX7 -- 7-segment displays
       : out std_logic_vector(6 downto 0);
    LEDG : out std_logic_vector(8 downto 0);       -- Green LEDs
    LEDR : out std_logic_vector(17 downto 0);      -- Red LEDs

    -- RS-232 interface

    UART_TXD : out std_logic;                      -- UART transmitter   
    UART_RXD : in std_logic;                       -- UART receiver

    -- IRDA interface

--    IRDA_TXD : out std_logic;                      -- IRDA Transmitter
    IRDA_RXD : in std_logic;                       -- IRDA Receiver

    -- SDRAM
   
    DRAM_DQ : inout std_logic_vector(15 downto 0); -- Data Bus
    DRAM_ADDR : out std_logic_vector(11 downto 0); -- Address Bus    
    DRAM_LDQM,                                     -- Low-byte Data Mask 
    DRAM_UDQM,                                     -- High-byte Data Mask
    DRAM_WE_N,                                     -- Write Enable
    DRAM_CAS_N,                                    -- Column Address Strobe
    DRAM_RAS_N,                                    -- Row Address Strobe
    DRAM_CS_N,                                     -- Chip Select
    DRAM_BA_0,                                     -- Bank Address 0
    DRAM_BA_1,                                     -- Bank Address 0
    DRAM_CLK,                                      -- Clock
    DRAM_CKE : out std_logic;                      -- Clock Enable

    -- FLASH
    
    FL_DQ : inout std_logic_vector(7 downto 0);      -- Data bus
    FL_ADDR : out std_logic_vector(21 downto 0);  -- Address bus
    FL_WE_N,                                         -- Write Enable
    FL_RST_N,                                        -- Reset
    FL_OE_N,                                         -- Output Enable
    FL_CE_N : out std_logic;                         -- Chip Enable

    -- SRAM
    
    SRAM_DQ : inout std_logic_vector(15 downto 0); -- Data bus 16 Bits
    SRAM_ADDR : out std_logic_vector(17 downto 0); -- Address bus 18 Bits
    SRAM_UB_N,                                     -- High-byte Data Mask 
    SRAM_LB_N,                                     -- Low-byte Data Mask 
    SRAM_WE_N,                                     -- Write Enable
    SRAM_CE_N,                                     -- Chip Enable
    SRAM_OE_N : out std_logic;                     -- Output Enable

    -- USB controller
    
    OTG_DATA : inout std_logic_vector(15 downto 0); -- Data bus
    OTG_ADDR : out std_logic_vector(1 downto 0);    -- Address
    OTG_CS_N,                                       -- Chip Select
    OTG_RD_N,                                       -- Write
    OTG_WR_N,                                       -- Read
    OTG_RST_N,                                      -- Reset
    OTG_FSPEED,                     -- USB Full Speed, 0 = Enable, Z = Disable
    OTG_LSPEED : out std_logic;     -- USB Low Speed, 0 = Enable, Z = Disable
    OTG_INT0,                                       -- Interrupt 0
    OTG_INT1,                                       -- Interrupt 1
    OTG_DREQ0,                                      -- DMA Request 0
    OTG_DREQ1 : in std_logic;                       -- DMA Request 1   
    OTG_DACK0_N,                                    -- DMA Acknowledge 0
    OTG_DACK1_N : out std_logic;                    -- DMA Acknowledge 1

    -- 16 X 2 LCD Module
    
    LCD_ON,                     -- Power ON/OFF
    LCD_BLON,                   -- Back Light ON/OFF
    LCD_RW,                     -- Read/Write Select, 0 = Write, 1 = Read
    LCD_EN,                     -- Enable
    LCD_RS : out std_logic;     -- Command/Data Select, 0 = Command, 1 = Data
    LCD_DATA : inout std_logic_vector(7 downto 0); -- Data bus 8 bits

    -- SD card interface
    
    SD_DAT,                     -- SD Card Data
    SD_DAT3,                    -- SD Card Data 3
    SD_CMD : inout std_logic;   -- SD Card Command Signal
    SD_CLK : out std_logic;     -- SD Card Clock

    -- USB JTAG link
    
    TDI,                        -- CPLD -> FPGA (data in)
    TCK,                        -- CPLD -> FPGA (clk)
    TCS : in std_logic;         -- CPLD -> FPGA (CS)
    TDO : out std_logic;        -- FPGA -> CPLD (data out)

    -- I2C bus
    
    I2C_SDAT : inout std_logic; -- I2C Data
    I2C_SCLK : out std_logic;   -- I2C Clock

    -- PS/2 port

    PS2_DAT,                    -- Data
    PS2_CLK : in std_logic;     -- Clock

    -- VGA output
    
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0);           -- Blue[9:0]

    --  Ethernet Interface
    
    ENET_DATA : inout std_logic_vector(15 downto 0);    -- DATA bus 16Bits
    ENET_CMD,           -- Command/Data Select, 0 = Command, 1 = Data
    ENET_CS_N,                                          -- Chip Select
    ENET_WR_N,                                          -- Write
    ENET_RD_N,                                          -- Read
    ENET_RST_N,                                         -- Reset
    ENET_CLK : out std_logic;                           -- Clock 25 MHz
    ENET_INT : in std_logic;                            -- Interrupt
    
    -- Audio CODEC
    
    AUD_ADCLRCK : inout std_logic;                      -- ADC LR Clock
    AUD_ADCDAT : in std_logic;                          -- ADC Data
    AUD_DACLRCK : inout std_logic;                      -- DAC LR Clock
    AUD_DACDAT : out std_logic;                         -- DAC Data
    AUD_BCLK : inout std_logic;                         -- Bit-Stream Clock
    AUD_XCK : out std_logic;                            -- Chip Clock
    
    -- Video Decoder
    
    TD_DATA : in std_logic_vector(7 downto 0);  -- Data bus 8 bits
    TD_HS,                                      -- H_SYNC
    TD_VS : in std_logic;                       -- V_SYNC
    TD_RESET : out std_logic;                   -- Reset
    
    -- General-purpose I/O
    
    GPIO_0,                                      -- GPIO Connection 0
    GPIO_1 : inout std_logic_vector(35 downto 0) -- GPIO Connection 1   
    );
  
end guitar_effects_top;

architecture datapath of guitar_effects_top is

 -- component de2_wm8731_audio_in is
 --  port (
 --   clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
 --   reset_n : in std_logic;
 --   data_out : out std_logic_vector(15 downto 0);
 -- 	audio_req : out std_logic;

    -- Audio interface signals
--	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
 --   AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
 --   AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
--  );
 -- end component;

 -- component de2_wm8731_audio_out is
 --  port (
 --   clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
 --   reset_n : in std_logic;
 --   test_mode : in std_logic;       --    Audio CODEC controller test mode
 --   data : in std_logic_vector(15 downto 0);
 -- 	audio_req : out std_logic;

    -- Audio interface signals
 --   AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
 --   AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
 --   AUD_DACDAT   : out std_logic   --    Audio CODEC DAC Data
 --   AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
 -- );
 -- end component;

--  component de2_i2c_av_config is
--  port (
--    iCLK : in std_logic;
--    iRST_N : in std_logic;
--    I2C_SCLK : out std_logic;
--    I2C_SDAT : inout std_logic
--  );
--  end component;

--  component edgedector is
-- port (
--	edge, clk: in std_logic;
--	P : out std_logic
--	);
--	end component;

--component distortion is
--port (
--	clk : in std_logic;
--	reset_n : in std_logic;
--	data_ready : in std_logic;
--	data_request : in std_logic;
--	selector : in std_logic;
	
--	data_in : in std_logic_vector(15 downto 0);
--	data_out : out std_logic_vector(15 downto 0);
--	clip : in std_logic_vector(15 downto 0)

--);
--end component;

--component test_module is
--port (
--    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
--    reset_n : in std_logic;

	-- for audio in
--    audio_send : in std_logic;
--   	data_in : in std_logic_vector(15 downto 0);

	-- for audio out
--    audio_req : in std_logic;
--    data_out : out std_logic_vector(15 downto 0)
--  );
-- end component;

--component mux is 
--port (
	--Inputs
--	data1_in : in std_logic_vector(15 downto 0);
--	data2_in : in std_logic_vector(15 downto 0);
	
	--Input from Switch Module
--	selector : in std_logic;
	
	--Output
--	dataout : out std_logic_vector(15 downto 0)
--);
--end component;

--component dist_button_ctl is
--port (
--	clk: in std_logic;
--	reset_n : in std_logic;
--	button_up : in std_logic;
--	button_down : in std_logic;

--	clip_value : out std_logic_vector (15 downto 0)
--	);
--end component;


--component vibrato_button_ctl is
--port (
--	clk: in std_logic;
--	reset_n : in std_logic;
--	button_up : in std_logic;
--	button_down : in std_logic;

--	value : out std_logic_vector (2 downto 0)
--	);
--end component;

--component vibrato_button_ctl2 is
--port (
--	clk: in std_logic;
--	reset_n : in std_logic;
--	button_up : in std_logic;
--	button_down : in std_logic;

--	value : out std_logic_vector (3 downto 0)
--	);
--end component;

--component effector is
--port (
--	clk : in std_logic;
--	audio_clk : in std_logic;
--	reset_n : in std_logic;

	-- for temporal testing... we will include this as bus commande from CPU!
--	distortion_enable : in std_logic;
--	clip : in std_logic_vector(15 downto 0);	

--    vibrato_enable: in std_logic;
--    vibrato_amp: in std_logic_vector(2 downto 0);
--	vibrato_freq: in std_logic_vector(3 downto 0);

--    chorus_enable: in std_logic;
--    chorus_amp: in std_logic_vector(2 downto 0);
--	chorus_freq: in std_logic_vector(3 downto 0);


--	vol_left_valid: in std_logic;
--	vol_left_data: in std_logic_vector(6 downto 0);
--	vol_right_valid: in std_logic;
--	vol_right_data: in std_logic_vector(6 downto 0);

    -- Audio interface signals
--	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
--  AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
--  AUD_BCLK     : inout std_logic;  --    Audio CODEC Bit-Stream Clock
--  AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
--  AUD_DACDAT   : out std_logic;   --    Audio CODEC DAC Data
--	I2C_SCLK	 : out std_logic;
--	I2C_SDAT	 : inout std_logic
--);
--end component;

  signal audio_clock : std_logic_vector(1 downto 0) := "00";
  signal value_from_dist_ctl_to_effector : std_logic_vector(15 downto 0);
  signal value_from_vibr_ctl_to_effector : std_logic_vector(2 downto 0);
  signal value_from_vibr_ctl_to_effector2 : std_logic_vector(3 downto 0);
--  signal audio_request : std_logic;
--  signal audio_send : std_logic;

--  signal from_ADC_to_dist : std_logic_vector (15 downto 0);
--  signal from_dist_to_DAC :std_logic_vector (15 downto 0);
--  signal from_selector_to_dist : std_logic;

--  signal from_clip_to_dist : std_logic_vector(15 downto 0);

begin

  process (CLOCK_50)
  begin
    if CLOCK_50'event and CLOCK_50 = '1' then
      audio_clock <= audio_clock + 1;
    end if;
  end process;

  AUD_XCK <= audio_clock(1);


--  V1: de2_wm8731_audio_in port map (
--    clk => audio_clock(1),
--    reset_n => '1',
--    data_out => from_ADC_to_dist,
--	audio_req => audio_send,  

    -- Audio interface signals
--    AUD_ADCLRCK  => AUD_ADCLRCK,
--    AUD_ADCDAT   => AUD_ADCDAT,
--    AUD_BCLK     => AUD_BCLK
--  );

--  M1:mux port map (
	--Inputs
--	data1_in => test_to_mux, 
--	data2_in => "0000000000000000",--dist_to_mux,
	
	--Input from Switch Module
--	selector => selector_to_mux,
	
	--Output
--	dataout => from_mux_to_DAC
--	);
--D1:dist_clipper port map (
--	clk => CLOCK_50,
--	reset_n => '1',
--	button_up => key(1), 
--	button_down => key(2),

--	clip_value => from_clip_to_dist
--	);


--T1: test_module port map(
--  clk => CLOCK_50,       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
--  reset_n => '1',

	-- for audio in
-- 	audio_send => audio_send,
--	data_in => from_ADC_to_modules,

	-- for audio out
-- 	audio_req => audio_request,
-- 	data_out => test_to_mux
--  );

--  T2: distortion port map (
--    clk => CLOCK_50,       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
--    reset_n => '1',

	-- for audio in
--    data_ready => audio_send,
--   	data_in => from_ADC_to_dist,

	-- for audio out
--    data_request => audio_request,
--    data_out => from_dist_to_DAC,
--	selector => from_selector_to_dist,
	--for clip value changes
--	clip => from_clip_to_dist
--  );

--  V2: de2_wm8731_audio_out port map (
--    clk => audio_clock(1),
--    reset_n => '1',
--    test_mode => '0',                   -- Output a sine wave
--    data => from_dist_to_DAC,
--	audio_req => audio_request,  


    -- Audio interface signals
--    AUD_ADCLRCK  => AUD_ADCLRCK,
--    AUD_DACDAT   => AUD_DACDAT,
--    AUD_DACLRCK  => AUD_DACLRCK
--    AUD_BCLK     => AUD_BCLK
--  );


--  K1: edgedector port map (
-- 	clk => CLOCK_50,
--	edge => key(0),
--	p => from_selector_to_dist
--	);

--CTL1: dist_button_ctl port map(
--	clk => CLOCK_50,
--	reset_n => SW(17),
--	button_up => KEY(0),
--	button_down => KEY(1),

--	clip_value => value_from_dist_ctl_to_effector
--);
--CTL2: vibrato_button_ctl port map(
--	clk => CLOCK_50,
--	reset_n => SW(17),
--	button_up => KEY(2),
--	button_down => KEY(3),

--	value => value_from_vibr_ctl_to_effector
--);

--CTL3: vibrato_button_ctl2 port map(
--	clk => CLOCK_50,
--	reset_n => SW(17),
--	button_up => KEY(0),
--	button_down => KEY(1),

--	value => value_from_vibr_ctl_to_effector2
--);


--E1: effector port map(
--	clk => CLOCK_50,
--	audio_clk => audio_clock(1),
--	reset_n => SW(17),

	-- for temporal testing... we will include this as bus commande from CPU!
--	distortion_enable => SW(0),
--	clip => value_from_dist_ctl_to_effector,

--    vibrato_enable => SW(1),
--    vibrato_amp => value_from_vibr_ctl_to_effector,
--	vibrato_freq => value_from_vibr_ctl_to_effector2,
	
--    chorus_enable => SW(2),
--    chorus_amp => value_from_vibr_ctl_to_effector,
--	chorus_freq => value_from_vibr_ctl_to_effector2,
	
	
--	Lvol_valid => '0',
--	Rvol_valid => '0',
--	Lvol_data => "0000000",
--	Rvol_data => "0000000",

    -- Audio interface signals
--	AUD_ADCLRCK  => AUD_ADCLRCK,
 --   AUD_ADCDAT   => AUD_ADCDAT,
  --  AUD_BCLK     => AUD_BCLK,
   -- AUD_DACLRCK  => AUD_DACLRCK,
    --AUD_DACDAT   => AUD_DACDAT,
	--I2C_SCLK	 => I2C_SCLK,
	--I2C_SDAT	 => I2C_SDAT
--);

  HEX7 <= "1111" & (not value_from_vibr_ctl_to_effector(2 downto 0));
  HEX6 <= "111"  & (not value_from_vibr_ctl_to_effector2(3 downto 0));
--  HEX5 <= value_from_vibr_ctl_to_effector(1 downto 0)& "00000";
--  HEX7     <= "1111000"; -- Leftmost
--  HEX6     <= "0000010";
  HEX5     <= "1111111";
  HEX4     <= "1111111";
  HEX3     <= "1111111";
  HEX2     <= "1111111";
 -- HEX4 <= value_from_vibr_ctl_to_effector2(15 downto 9);
 -- HEX3 <= value_from_vibr_ctl_to_effector2(8 downto 2);
 -- HEX2 <= value_from_vibr_ctl_to_effector2(1 downto 0)& "00000";
  HEX1     <= "1111111";
  HEX0     <= "1111111";          -- Rightmost
  LEDG(7 downto 0) 	   <= "11111111";
  LEDR(17)     <= (not SW(17));
  LEDR(0)     <= SW(0);
  LEDR(1)     <= SW(1);
  LEDR(2)	  <= SW(2);
  LEDR(16 downto 3) <= (others => '1');
  LCD_ON   <= '1';
  LCD_BLON <= '1';

  -- Set all bidirectional ports to tri-state
  DRAM_DQ     <= (others => 'Z');
  FL_DQ       <= (others => 'Z');
  SRAM_DQ     <= (others => 'Z');
  OTG_DATA    <= (others => 'Z');
  LCD_DATA    <= (others => 'Z');
  SD_DAT      <= 'Z';
  I2C_SDAT    <= 'Z';
  ENET_DATA   <= (others => 'Z');
  GPIO_0      <= (others => 'Z');
  GPIO_1      <= (others => 'Z');

end datapath;
